/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.dql.connector;

import java.util.Date;
import java.util.List;
import java.util.Properties;

import tools.descartes.dql.models.mapping.mapping.EntityMapping;

/**
 * This interface defines the necessary methods to execute queries for
 * performance metrics-related information. The information retrieved can be
 * used to analyze the performance aspects of a performance model instance.
 * 
 * @author Fabian Gorsler <fabian.gorsler@student.kit.edu>
 */
public interface PerformanceMetricsQueryConnector extends StatefulQueryConnector {
	/**
	 * Specifies the key used in DoF Variation for intervals as start value. The
	 * key is used for the "iterate" function.
	 */
	public static final String DOF_VARIATION_START_KEY = "start";

	/**
	 * Specifies the key used in DoF Variation for intervals as ending value.
	 * The key is used for the "iterate" function.
	 */
	public static final String DOF_VARIATION_END_KEY = "end";

	/**
	 * Specifies the key used in DoF Variation for intervals as step width
	 * between start and end. The key is used for the "iterate" function.
	 */
	public static final String DOF_VARIATION_STEP_KEY = "step";

	/**
	 * Specifies the key used in DoF Variation for intervals as ending value.
	 * The key is used for the "iterate" function.
	 */
	public static final String DOF_VARIATION_FUNCTION_KEY = "function";

	/**
	 * Specifies the value used in DoF Variation for calling the iterate
	 * function.
	 */
	public static final String DOF_VARIATION_INTERVAL_FUNCTION_VALUE = "iterate";

	/**
	 * Specifies the key used in DoF Variation for naming the function to use a
	 * fixed set of values.
	 */
	public static final String DOF_VARIATION_VALUE_FUNCTION_VALUE = "fixed";

	/**
	 * Specifies the key used in DoF Variation to identify a fixed set of
	 * values.
	 */
	public static final String DOF_VARIATION_VALUE_KEY = "value";

	/**
	 * Get the name of the constraint currently set
	 * 
	 * @return The name of the constraint or null, if no constraint is set
	 */
	public String getConstraint();

	/**
	 * Get the name of the constraint set for DoF exploration
	 * 
	 * @return The name of the constraint set for DoF exploration or null, if no
	 *         constraint is set
	 */
	public String getDoFConstraint();

	/**
	 * Get the name of the exploration strategy currently chosen
	 * 
	 * @return The name of the exploration strategy or null, if no exploration
	 *         strategy is set
	 */
	public String getDoFExplorationStrategy();

	/**
	 * Get the properties of the exploration strategy currently chosen
	 * 
	 * @return The properties chosen or null, if no properties are set
	 */
	public Properties getDoFExplorationStrategyProperties();

	/**
	 * Get the list of supported constraint names
	 * 
	 * @return A list of constraint names
	 */
	public List<String> getSupportedConstraintNames();

	/**
	 * Get the list of supported DoF constraint names
	 * 
	 * @return A list of constraint names
	 */
	public List<String> getSupportedDoFConstraintNames();

	/**
	 * Get the list of supported exploration strategy names
	 * 
	 * @return A list of exploration strategy names
	 */
	public List<String> getSupportedExplorationStrategyNames();

	/**
	 * If a connector supports custom time units, they are returned using this
	 * method.
	 * 
	 * @return Supported connector-specific time units or null if no special
	 *         time units are supported
	 */
	public List<String> getSupportedTimeUnits();

	/**
	 * Request the connector to compute and solve all requested probes for the
	 * whole parameter space defined by the degrees of freedom
	 * 
	 * @see #requestMetrics(EntityMapping)
	 * 
	 * @param request
	 *            A mapping model consisting of model entities and probes to be
	 *            computed
	 * @return A list of results for each computation in the parameter space
	 */
	public List<EntityMapping> requestDoFMetrics(EntityMapping request);

	/**
	 * Request the connector to compute and solve all requested probes for the
	 * model entities in the mapping model instance
	 * 
	 * @param request
	 *            A mapping model consisting of model entities and probes to be
	 *            computed
	 * @return A mapping model instance with computed metrics for specified
	 *         probes
	 */
	public EntityMapping requestMetrics(EntityMapping request);

	/**
	 * Set a constraint on the computation of metrics to be accurate or fast
	 * 
	 * @param constraint
	 *            The constraint for the computation
	 */
	public void setConstraint(String constraint);

	/**
	 * Set a constraint on the computation of degrees of freedom to explore the
	 * parameter space fast or accurate
	 * 
	 * @param doFConstraint
	 *            The constraint for the exploration
	 */
	public void setDoFConstraint(String doFConstraint);

	/**
	 * Set the name of the exploration strategy used for parameter space
	 * exploration in a query with degrees of freedom
	 * 
	 * @see #getSupportedExplorationStrategyNames()
	 * 
	 * @param strategy
	 *            The name of an exploration strategy provided by the Connector
	 * @param properties
	 *            Properties for the dynamic configuration of the used strategy
	 *            or null if not used
	 */
	public void setDoFExplorationStrategy(String strategy, Properties properties);

	/**
	 * Set the duration of an observation period by defining its start and end
	 * time stamp as {@link java.util.Date}.
	 * 
	 * @param start
	 *            A Date instance specifying the start of an observation period
	 * @param end
	 *            A Date instance specifying the end of an observation period
	 * @param aggregation
	 *            If not null, sets the time span to aggregate values.
	 *            Interpretation and aggregation is connector-specific.
	 */
	public void setObservationPeriod(Date start, Date end, long sampleSize);

	/**
	 * Sets a connector-specific observation period starting at the current
	 * point of time. Available time units can be retrieved using
	 * #getTimeUnits().
	 * 
	 * @param amount
	 * @param unit
	 * @param sampleSize
	 */
	public void setObservationPeriod(long amount, String unit, long sampleSize);

	/**
	 * Sets a connector-specific model reference to be used as data source for
	 * the incoming query. The type of reference supplied is defined by the
	 * connector.
	 * 
	 * @param type
	 *            The type of reference to be used
	 * @param identifier
	 *            The reference identifier
	 * @param sampleSize
	 *            The size of samples, if applicable
	 */
	public void setObservationReference(String type, String identifier, long sampleSize);

	/**
	 * Test whether constrained requests are supported by the implementing class
	 * 
	 * @return True if entity requests are supported, false otherwise
	 */
	public boolean supportsConstraintRequests();

	/**
	 * Test whether degree of freedom requests are supported by the implementing
	 * class
	 * 
	 * @return True if degree of freedom requests are supported, false otherwise
	 */
	public boolean supportsDoFRequests();

	/**
	 * Test whether metric requests are supported by the implementing class
	 * 
	 * @return True if entity requests are supported, false otherwise
	 */
	public boolean supportsMetricRequests();

	/**
	 * Tests whether observation periods are supported by the implementing class
	 * 
	 * @return True if observation periods are supported, false otherwise
	 */
	public boolean supportsObservationPeriods();
	
}


	