/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.pavo.types;

import java.awt.Color;
import java.math.BigDecimal;

import javax.swing.JPanel;
import javax.swing.JRadioButton;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.ValueMarker;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYErrorRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.data.xy.YIntervalSeries;
import org.jfree.data.xy.YIntervalSeriesCollection;
import org.jfree.ui.RectangleAnchor;
import org.jfree.ui.TextAnchor;

import tools.descartes.pavo.controller.VisualizationController;
import tools.descartes.pavo.result.AbstractQuantitativeResult;
import tools.descartes.pavo.result.ResultContainerPavo;
import tools.descartes.pavo.result.SeriesResultPavo;
import tools.descartes.pavo.result.ValueResultPavo;
import tools.descartes.pavo.util.Point;
import tools.descartes.pavo.view.View;

/**
 * Class to visualise the current result as LineChart if possible. Can visualise
 * results constiting out of value results or series results (Values are shown
 * as horizontal markers). Does not do any configurations for the control
 * buttons
 * 
 * @author Maximilian König <max.c.koenig@t-online.de>
 * @since 2016
 */
public class LineChart extends AbstractVisualizationType implements IVisualizationType {

	public LineChart(ResultContainerPavo resultN) {
		super(resultN);
	}

	@Override
	public JFreeChart visualize() {

		XYSeriesCollection dataset = new XYSeriesCollection();
		XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
		NumberAxis domainAxis = new NumberAxis(this.getXDescription());
		domainAxis.setAutoRangeIncludesZero(false);
		NumberAxis valueAxis = new NumberAxis(this.getYDescription());
		XYPlot plot = new XYPlot(dataset, domainAxis, valueAxis, renderer);
		JFreeChart chart = new JFreeChart("", JFreeChart.DEFAULT_TITLE_FONT, plot, true);
		int i = 0;
		for (AbstractQuantitativeResult r : this.getResult().getResults()) {
			if (r instanceof ValueResultPavo) {
				ValueMarker marker = new ValueMarker(((ValueResultPavo) r).getValue());
				marker.setPaint(Color.green);
				marker.setLabel(r.getMetric() + " " + i);
				i++;
				marker.setLabelAnchor(RectangleAnchor.BOTTOM_RIGHT);
				marker.setLabelTextAnchor(TextAnchor.TOP_RIGHT);
				((XYPlot) chart.getPlot()).addRangeMarker(marker);
				ValueAxis yAxis = plot.getRangeAxis();
				if (marker.getValue() > yAxis.getUpperBound())
					yAxis.setRange(yAxis.getLowerBound(), marker.getValue() + 1);
			} else {
				XYSeries series1 = new XYSeries(((SeriesResultPavo) r).getSeriesName());
				for (Point p : ((SeriesResultPavo) r).getValues()) {
					series1.add(p.getX(), p.getY());
				}
				dataset.addSeries(series1);
			}

		}
		plot.setBackgroundPaint(Color.white);
		plot.setRangeGridlinePaint(Color.lightGray);
		plot.setDomainGridlinePaint(Color.lightGray);
		return chart;
	}

	@Override
	public boolean canVisualize() {
		for (AbstractQuantitativeResult result : this.getResult().getResults()) {
			if (!(result instanceof ValueResultPavo)
					&& !(result instanceof SeriesResultPavo && ((SeriesResultPavo) result).isContinous())) {
				return false;
			}
		}
		return true;
	}

	public JFreeChart visualizeConfidence() {
		YIntervalSeriesCollection dataset = new YIntervalSeriesCollection();
		NumberAxis xAxis = new NumberAxis(this.getXDescription());
		NumberAxis yAxis = new NumberAxis(this.getYDescription());
		XYErrorRenderer renderer = new XYErrorRenderer();
		renderer.setBaseLinesVisible(true);
		renderer.setBaseShapesVisible(false);
		XYPlot plot = new XYPlot(dataset, xAxis, yAxis, renderer);
		JFreeChart chart = new JFreeChart("", plot);
		for (AbstractQuantitativeResult r : this.getResult().getResults()) {
			if (r instanceof ValueResultPavo) {
				ValueMarker marker = new ValueMarker(((ValueResultPavo) r).getValue());
				marker.setPaint(Color.green);
				marker.setLabel(((ValueResultPavo) r).getName());
				marker.setLabelAnchor(RectangleAnchor.BOTTOM_RIGHT);
				marker.setLabelTextAnchor(TextAnchor.TOP_RIGHT);
				((XYPlot) chart.getPlot()).addRangeMarker(marker);
				if (marker.getValue() > yAxis.getUpperBound())
					yAxis.setRange(yAxis.getLowerBound(), marker.getValue() + 1);
			} else {
				YIntervalSeries s1 = new YIntervalSeries(((SeriesResultPavo) r).getSeriesName());
				for (Point p : ((SeriesResultPavo) r).getValues()) {
					BigDecimal conf;
					if (p.getConf() == null)
						conf = BigDecimal.valueOf(0.0);
					else
						conf = p.getConf().getIntervalHalfLength();

					Double x = p.getX();
					Double y = p.getY();
					s1.add(x, y, y - conf.doubleValue(), y + conf.doubleValue());
				}
				dataset.addSeries(s1);
			}

		}
		plot.setBackgroundPaint(Color.white);
		plot.setRangeGridlinePaint(Color.lightGray);
		plot.setDomainGridlinePaint(Color.lightGray);
		return chart;
	}

	public JPanel getEditViewComponent(VisualizationController c) {
		JPanel panel = new JPanel();
		JRadioButton a = View.createConfidence(c.getActionListenerConfidence());
		panel.add(a);

		try {
			if (c.getView().getConfidenceRadio().isSelected()) {
				a.setSelected(true);
			}
		} catch (NullPointerException e) {

		}

		c.getView().addConfidence(a);
		panel.add(View.createMinMaxMean(c.createActionListenerMin(), c.createActionListenerMax(),
				c.createActionListenerMean()));
		return panel;
	}


}